==> js-language-23.1.0-sources.jar: com/oracle/truffle/js/parser/json/JSONParserUtil.java <==
/*
 * Copyright (c) 2018, 2020, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * The Universal Permissive License (UPL), Version 1.0
 *
 * Subject to the condition set forth below, permission is hereby granted to any
 * person obtaining a copy of this software, associated documentation and/or
 * data (collectively the "Software"), free of charge and under any and all
 * copyright rights in the Software, and any and all patent rights owned or
 * freely licensable by each licensor hereunder covering either (i) the
 * unmodified Software as contributed to or provided by such licensor, or (ii)
 * the Larger Works (as defined below), to deal in both
 *
 * (a) the Software, and
 *
 * (b) any piece of software and/or hardware listed in the lrgrwrks.txt file if
 * one is included with the Software each a "Larger Work" to which the Software
 * is contributed by such licensors),
 *
 * without restriction, including without limitation the rights to copy, create
 * derivative works of, display, perform, and distribute the Software and make,
 * use, sell, offer for sale, import, export, have made, and have sold the
 * Software and the Larger Work(s), and to sublicense the foregoing rights on
 * either these or other terms.
 *
 * This license is subject to the following condition:
 *
 * The above copyright notice and either this complete permission notice or at a
 * minimum a reference to the UPL must be included in all copies or substantial
 * portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package com.oracle.truffle.js.parser.json;

import com.oracle.truffle.api.strings.TruffleString;
import com.oracle.truffle.api.strings.TruffleStringBuilder;
import com.oracle.truffle.js.runtime.Strings;

public class JSONParserUtil {

    /**
     * Implementation of the Quote(value) operation as defined in the ECMAscript spec. It wraps a
     * String value in double quotes and escapes characters within.
     *
     * @param value string to quote
     *
     * @return quoted and escaped string
     */
    public static TruffleString quote(final TruffleString value) {
        TruffleStringBuilder product = Strings.builderCreate();
        Strings.builderAppend(product, Strings.DOUBLE_QUOTE);
        int len = Strings.length(value);
        for (int i = 0; i < len; i++) {
            char ch = Strings.charAt(value, i);
            if (ch < ' ') {
                if (ch == '\b') {
                    Strings.builderAppend(product, Strings.BACKSLASH_B);
                } else if (ch == '\f') {
                    Strings.builderAppend(product, Strings.BACKSLASH_F);
                } else if (ch == '\n') {
                    Strings.builderAppend(product, Strings.BACKSLASH_N);
                } else if (ch == '\r') {
                    Strings.builderAppend(product, Strings.BACKSLASH_R);
                } else if (ch == '\t') {
                    Strings.builderAppend(product, Strings.BACKSLASH_T);
                } else {
                    Strings.builderAppend(product, Strings.BACKSLASH_U);
                    TruffleString hex = Strings.intToHexString(ch);
                    for (int j = Strings.length(hex); j < 4; j++) {
                        Strings.builderAppend(product, '0');
                    }
                    Strings.builderAppend(product, hex);
                }
            } else {
                if (ch == '\\') {
                    Strings.builderAppend(product, Strings.BACKSLASH_BACKSLASH);
                } else if (ch == '"') {
                    Strings.builderAppend(product, Strings.BACKSLASH_DOUBLE_QUOTE);
                } else {
                    Strings.builderAppend(product, ch);
                }
            }
        }
        Strings.builderAppend(product, Strings.DOUBLE_QUOTE);
        return Strings.builderToString(product);
    }
}

==> js-language-24.0.1-sources.jar: com/oracle/truffle/js/parser/json/JSONParserUtil.java <==
/*
 * Copyright (c) 2018, 2023, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * The Universal Permissive License (UPL), Version 1.0
 *
 * Subject to the condition set forth below, permission is hereby granted to any
 * person obtaining a copy of this software, associated documentation and/or
 * data (collectively the "Software"), free of charge and under any and all
 * copyright rights in the Software, and any and all patent rights owned or
 * freely licensable by each licensor hereunder covering either (i) the
 * unmodified Software as contributed to or provided by such licensor, or (ii)
 * the Larger Works (as defined below), to deal in both
 *
 * (a) the Software, and
 *
 * (b) any piece of software and/or hardware listed in the lrgrwrks.txt file if
 * one is included with the Software each a "Larger Work" to which the Software
 * is contributed by such licensors),
 *
 * without restriction, including without limitation the rights to copy, create
 * derivative works of, display, perform, and distribute the Software and make,
 * use, sell, offer for sale, import, export, have made, and have sold the
 * Software and the Larger Work(s), and to sublicense the foregoing rights on
 * either these or other terms.
 *
 * This license is subject to the following condition:
 *
 * The above copyright notice and either this complete permission notice or at a
 * minimum a reference to the UPL must be included in all copies or substantial
 * portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package com.oracle.truffle.js.parser.json;

import com.oracle.truffle.api.strings.TruffleString;
import com.oracle.truffle.js.runtime.Strings;

public class JSONParserUtil {

    /**
     * Implementation of the Quote(value) operation as defined in the ECMAscript spec. It wraps a
     * String value in double quotes and escapes characters within.
     *
     * @param value string to quote
     *
     * @return quoted and escaped string
     */
    public static TruffleString quote(final TruffleString value) {
        var product = Strings.builderCreate();
        Strings.builderAppend(product, Strings.DOUBLE_QUOTE);
        int len = Strings.length(value);
        for (int i = 0; i < len; i++) {
            char ch = Strings.charAt(value, i);
            if (ch < ' ') {
                if (ch == '\b') {
                    Strings.builderAppend(product, Strings.BACKSLASH_B);
                } else if (ch == '\f') {
                    Strings.builderAppend(product, Strings.BACKSLASH_F);
                } else if (ch == '\n') {
                    Strings.builderAppend(product, Strings.BACKSLASH_N);
                } else if (ch == '\r') {
                    Strings.builderAppend(product, Strings.BACKSLASH_R);
                } else if (ch == '\t') {
                    Strings.builderAppend(product, Strings.BACKSLASH_T);
                } else {
                    Strings.builderAppend(product, Strings.BACKSLASH_U);
                    TruffleString hex = Strings.intToHexString(ch);
                    for (int j = Strings.length(hex); j < 4; j++) {
                        Strings.builderAppend(product, '0');
                    }
                    Strings.builderAppend(product, hex);
                }
            } else {
                if (ch == '\\') {
                    Strings.builderAppend(product, Strings.BACKSLASH_BACKSLASH);
                } else if (ch == '"') {
                    Strings.builderAppend(product, Strings.BACKSLASH_DOUBLE_QUOTE);
                } else {
                    Strings.builderAppend(product, ch);
                }
            }
        }
        Strings.builderAppend(product, Strings.DOUBLE_QUOTE);
        return Strings.builderToString(product);
    }
}
